module axi_lite_slave_32
(
  input  clk,
  input  reset,
  output [32-1 : 0] data_length,
  input [32-1 : 0] data_address,
  input [32-1 : 0] data_din,
  output [32-1 : 0] data_dout,
  input  data_we,
  input  data_oe,
  input [32-1 : 0] axi_reader_ARADDR,
  input  axi_reader_ARVALID,
  output  axi_reader_ARREADY,
  output [32-1 : 0] axi_reader_RDATA,
  output [2-1 : 0] axi_reader_RRESP,
  output  axi_reader_RVALID,
  input  axi_reader_RREADY,
  input [32-1 : 0] axi_writer_AWADDR,
  input  axi_writer_AWVALID,
  output  axi_writer_AWREADY,
  input [32-1 : 0] axi_writer_WDATA,
  input  axi_writer_WVALID,
  output  axi_writer_WREADY,
  input [4-1 : 0] axi_writer_WSTRB,
  output [2-1 : 0] axi_writer_BRESP,
  output  axi_writer_BVALID,
  input  axi_writer_BREADY
);

  wire  clk_sig;
  wire  reset_sig;
  wire [32-1 : 0] data_length_sig;
  wire [32-1 : 0] data_address_sig;
  wire [32-1 : 0] data_din_sig;
  wire [32-1 : 0] data_dout_sig;
  wire  data_we_sig;
  wire  data_oe_sig;
  wire [32-1 : 0] axi_reader_ARADDR_sig;
  wire  axi_reader_ARVALID_sig;
  reg  axi_reader_ARREADY_sig = 1'b0;
  reg [32-1 : 0] axi_reader_RDATA_sig = 0;
  reg [2-1 : 0] axi_reader_RRESP_sig = 0;
  reg  axi_reader_RVALID_sig = 1'b0;
  wire  axi_reader_RREADY_sig;
  wire [32-1 : 0] axi_writer_AWADDR_sig;
  wire  axi_writer_AWVALID_sig;
  reg  axi_writer_AWREADY_sig = 1'b0;
  wire [32-1 : 0] axi_writer_WDATA_sig;
  wire  axi_writer_WVALID_sig;
  reg  axi_writer_WREADY_sig = 1'b0;
  wire [4-1 : 0] axi_writer_WSTRB_sig;
  reg [2-1 : 0] axi_writer_BRESP_sig = 0;
  reg  axi_writer_BVALID_sig = 1'b0;
  wire  axi_writer_BREADY_sig;

  reg signed [32-1 : 0] data_address_b = 0;
  reg signed [32-1 : 0] data_din_b = 0;
  wire signed [32-1 : 0] data_dout_b;
  reg  data_we_b = 1'b0;
  reg  data_oe_b = 1'b0;
  wire  U_clk;
  wire  U_reset;
  wire signed [32-1 : 0] U_length;
  wire signed [32-1 : 0] U_address;
  wire signed [32-1 : 0] U_din;
  wire signed [32-1 : 0] U_dout;
  wire  U_we;
  wire  U_oe;
  wire signed [32-1 : 0] U_address_b;
  wire signed [32-1 : 0] U_din_b;
  wire signed [32-1 : 0] U_dout_b;
  wire  U_we_b;
  wire  U_oe_b;
  localparam write_seq_IDLE = 32'd0;
  localparam write_seq_s0 = 32'd1;
  localparam write_seq_s1 = 32'd2;
  localparam write_seq_s2 = 32'd3;
  reg [31:0] write_seq = write_seq_IDLE;
  reg signed [32-1 : 0] write_seq_delay = 0;
  wire [32-1 : 0] tmp_0001;
  wire  tmp_0002;
  wire  tmp_0003;
  wire [32-1 : 0] tmp_0004;


  assign clk_sig = clk;
  assign reset_sig = reset;
  assign data_length = data_length_sig;
  assign data_length_sig = U_length;

  assign data_address_sig = data_address;
  assign data_din_sig = data_din;
  assign data_dout = data_dout_sig;
  assign data_dout_sig = U_dout;

  assign data_we_sig = data_we;
  assign data_oe_sig = data_oe;
  assign axi_reader_ARADDR_sig = axi_reader_ARADDR;
  assign axi_reader_ARVALID_sig = axi_reader_ARVALID;
  assign axi_reader_ARREADY = axi_reader_ARREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_ARREADY_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_reader_ARREADY_sig <= tmp_0003;
      end
    end
  end

  assign axi_reader_RDATA = axi_reader_RDATA_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_RDATA_sig <= 0;
    end else begin
      if (write_seq == write_seq_s2) begin
        axi_reader_RDATA_sig <= data_dout_b;
      end
    end
  end

  assign axi_reader_RRESP = axi_reader_RRESP_sig;
  assign axi_reader_RVALID = axi_reader_RVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_RVALID_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_s2) begin
        axi_reader_RVALID_sig <= 1'b1;
      end else begin
        axi_reader_RVALID_sig <= 1'b0;
      end
    end
  end

  assign axi_reader_RREADY_sig = axi_reader_RREADY;
  assign axi_writer_AWADDR_sig = axi_writer_AWADDR;
  assign axi_writer_AWVALID_sig = axi_writer_AWVALID;
  assign axi_writer_AWREADY = axi_writer_AWREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_AWREADY_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_AWREADY_sig <= axi_writer_AWVALID_sig;
      end else begin
        axi_writer_AWREADY_sig <= 1'b0;
      end
    end
  end

  assign axi_writer_WDATA_sig = axi_writer_WDATA;
  assign axi_writer_WVALID_sig = axi_writer_WVALID;
  assign axi_writer_WREADY = axi_writer_WREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_WREADY_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_s0) begin
        axi_writer_WREADY_sig <= axi_writer_WVALID_sig;
      end else begin
        axi_writer_WREADY_sig <= 1'b0;
      end
    end
  end

  assign axi_writer_WSTRB_sig = axi_writer_WSTRB;
  assign axi_writer_BRESP = axi_writer_BRESP_sig;
  assign axi_writer_BVALID = axi_writer_BVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_BVALID_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_BVALID_sig <= axi_writer_BREADY_sig;
      end
    end
  end

  assign axi_writer_BREADY_sig = axi_writer_BREADY;

  assign tmp_0001 = {2'b0,axi_writer_AWADDR_sig[31:2]};
  assign tmp_0002 = ~axi_writer_AWVALID_sig;
  assign tmp_0003 = axi_reader_ARVALID_sig & tmp_0002;
  assign tmp_0004 = {2'b0,axi_reader_ARADDR_sig[31:2]};

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      write_seq <= write_seq_IDLE;
      write_seq_delay <= 32'h0;
    end else begin
      case (write_seq)
        write_seq_IDLE : begin
          if (axi_writer_AWVALID_sig == 1'b1) begin
            write_seq <= write_seq_s0;
          end else if (tmp_0003 == 1'b1) begin
            write_seq <= write_seq_s1;
          end
        end
        write_seq_s0 : begin
          if (axi_writer_WVALID_sig == 1'b1) begin
            write_seq <= write_seq_IDLE;
          end
        end
        write_seq_s1 : begin
          write_seq <= write_seq_s2;
        end
        write_seq_s2 : begin
          if (axi_reader_RREADY_sig == 1'b1) begin
            write_seq <= write_seq_IDLE;
          end
        end
      endcase
    end
  end


  always @(posedge clk) begin
    if(reset == 1'b1) begin
      data_address_b <= 0;
    end else begin
      if (write_seq == write_seq_IDLE && axi_writer_AWVALID_sig == 1'b1) begin
        data_address_b <= tmp_0001;
      end else if (write_seq == write_seq_IDLE && tmp_0003 == 1'b1) begin
        data_address_b <= tmp_0004;
      end
    end
  end

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      data_din_b <= 0;
    end else begin
      if (write_seq == write_seq_s0 && axi_writer_WVALID_sig == 1'b1) begin
        data_din_b <= axi_writer_WDATA_sig;
      end
    end
  end

  assign data_dout_b = U_dout_b;

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      data_we_b <= 1'b0;
    end else begin
      if (write_seq == write_seq_s0) begin
        data_we_b <= axi_writer_WVALID_sig;
      end else begin
        data_we_b <= 1'b0;
      end
    end
  end

  assign U_clk = clk_sig;

  assign U_reset = reset_sig;

  assign U_address = data_address_sig;

  assign U_din = data_din_sig;

  assign U_we = data_we_sig;

  assign U_oe = data_oe_sig;

  assign U_address_b = data_address_b;

  assign U_din_b = data_din_b;

  assign U_we_b = data_we_b;

  assign U_oe_b = data_oe_b;


  dualportram
  #(
    .WIDTH(32),
    .DEPTH(10),
    .WORDS(1024)
  )
  U
  (
    .clk(U_clk),
    .reset(U_reset),
    .length(U_length),
    .address(U_address),
    .din(U_din),
    .dout(U_dout),
    .we(U_we),
    .oe(U_oe),
    .address_b(U_address_b),
    .din_b(U_din_b),
    .dout_b(U_dout_b),
    .we_b(U_we_b),
    .oe_b(U_oe_b)
  );


endmodule
